searchNodes=[{"doc":"Noether aims to ease common data manipulation tasks by introducing simple algebraic functions and other utilities. Functions and names are inspired (sometimes taken as-is) from Haskell.The Noether.Maybe module introduces operations on nullable values.The Noether.Either module introduces operations on {:ok, _} | {:error, _} values.The Noether.List module introduces operations on lists.The root module has a few simple functions one might find of use.","ref":"Noether.html","title":"Noether","type":"module"},{"doc":"Takes a tuple and a function of arity 2. It applies the two values in the tuple to the function.Examplesiex&gt; curry({1, 2}, &amp;Kernel.+/2) 3","ref":"Noether.html#curry/2","title":"Noether.curry/2","type":"function"},{"doc":"Takes a function of arity 2 and returns the same function with its arguments in reverse order, i.e., &quot;flipped&quot;. Please note that if a function of different arity is given, a function of arity 2 is returned where the two arguments will be applied to the given function.Examplesiex&gt; flip(&amp;Kernel.-/2).(3, 4) 1","ref":"Noether.html#flip/1","title":"Noether.flip/1","type":"function"},{"doc":"Takes two values and applies them to a function or arity 1 in form of a tuple.Examplesiex&gt; uncurry(1, 2, &amp;(&amp;1)) {1, 2}","ref":"Noether.html#uncurry/3","title":"Noether.uncurry/3","type":"function"},{"doc":"","ref":"Noether.html#t:fun1/0","title":"Noether.fun1/0","type":"type"},{"doc":"","ref":"Noether.html#t:fun2/0","title":"Noether.fun2/0","type":"type"},{"doc":"This module hosts several utility functions to work with {:ok, _} | {:error, _} values. These type of values will be then called Either.","ref":"Noether.Either.html","title":"Noether.Either","type":"module"},{"doc":"Given an {:ok, value} and a function that returns an Either value, it applies the function on the value. It effectively &quot;squashes&quot; an {:ok, {:ok, v}} or {:ok, {:error, _}} to its most appropriate representation. If an {:error, _} is given, it is returned as-is.Examplesiex&gt; bind({:ok, 1}, fn a -&gt; {:ok, a + 1} end) {:ok, 2} iex&gt; bind({:ok, 1}, fn _ -&gt; {:error, 5} end) {:error, 5} iex&gt; bind({:error, 1}, fn _ -&gt; {:ok, 45} end) {:error, 1}","ref":"Noether.Either.html#bind/2","title":"Noether.Either.bind/2","type":"function"},{"doc":"Given a list of Either, the function is mapped only on the elements of type {:ok, _}. Other values will be discarded. A list of the results is returned outside of the tuple.Examplesiex&gt; cat_either([{:ok, 1}], &amp;(&amp;1 + 1)) [2] iex&gt; cat_either([{:ok, 1}, {:error, 2}, {:ok, 3}], &amp;(&amp;1 + 1)) [2, 4]","ref":"Noether.Either.html#cat_either/2","title":"Noether.Either.cat_either/2","type":"function"},{"doc":"Given a value and two functions that return an Either, it applies the first one and returns the result if it matches {:ok, _}. Otherwise the second function is applied.Examplesiex&gt; choose(0, fn a -&gt; {:ok, a + 1} end, fn b -&gt; {:ok, b + 2} end) {:ok, 1} iex&gt; choose(0, fn _ -&gt; {:error, 1} end, fn b -&gt; {:ok, b + 2} end) {:ok, 2} iex&gt; choose(0, fn _ -&gt; {:error, 1} end, fn _ -&gt; {:error, 2} end) {:error, 2}","ref":"Noether.Either.html#choose/3","title":"Noether.Either.choose/3","type":"function"},{"doc":"Given an Either and two functions, it applies the first or second one on the second value of the tuple, depending if the value is {:ok, _} or {:error, _} respectively.Examplesiex&gt; either({:ok, 1}, &amp;(&amp;1 + 1), &amp;(&amp;1 + 2)) {:ok, 2} iex&gt; either({:error, 1}, &amp;(&amp;1 + 1), &amp;(&amp;1 + 2)) {:error, 3}","ref":"Noether.Either.html#either/3","title":"Noether.Either.either/3","type":"function"},{"doc":"It returns true only if the value given matches a {:error, value} type.Examplesiex&gt; error?({:ok, 1}) false iex&gt; error?({:error, 2}) true iex&gt; error?(3) false","ref":"Noether.Either.html#error?/1","title":"Noether.Either.error?/1","type":"function"},{"doc":"Alias for bind/2Examplesiex&gt; flat_map({:ok, 1}, fn a -&gt; {:ok, a + 1} end) {:ok, 2} iex&gt; flat_map({:ok, 1}, fn _ -&gt; {:error, 5} end) {:error, 5} iex&gt; flat_map({:error, 1}, fn _ -&gt; {:ok, 45} end) {:error, 1}","ref":"Noether.Either.html#flat_map/2","title":"Noether.Either.flat_map/2","type":"function"},{"doc":"Alias for join/1.Examplesiex&gt; flatten({:ok, {:ok, 1}}) {:ok, 1} iex&gt; flatten({:ok, 1}) ** (FunctionClauseError) no function clause matching in Noether.Either.join/1 iex&gt; flatten({:error, &quot;Value not found&quot;}) {:error, &quot;Value not found&quot;}","ref":"Noether.Either.html#flatten/1","title":"Noether.Either.flatten/1","type":"function"},{"doc":"Given an {:ok, {:ok, value}} it flattens the ok unwrapping the value and returning {:ok, value}. If an {:error, _} is given, it is returned as-is.Examplesiex&gt; join({:ok, {:ok, 1}}) {:ok, 1} iex&gt; join({:ok, 1}) ** (FunctionClauseError) no function clause matching in Noether.Either.join/1 iex&gt; join({:error, &quot;Value not found&quot;}) {:error, &quot;Value not found&quot;}","ref":"Noether.Either.html#join/1","title":"Noether.Either.join/1","type":"function"},{"doc":"Given an {:ok, value} and a function, it applies the function on the value returning {:ok, f.(value)}. If an {:error, _} is given, it is returned as-is.Examplesiex&gt; map({:ok, -1}, &amp;Kernel.abs/1) {:ok, 1} iex&gt; map({:error, &quot;Value not found&quot;}, &amp;Kernel.abs/1) {:error, &quot;Value not found&quot;}","ref":"Noether.Either.html#map/2","title":"Noether.Either.map/2","type":"function"},{"doc":"Given a list of values and a function returning {:ok, any} or {:error, any}, it applies the function on every value returning {:ok, values} if every f.(v) results in {:ok, v}; returning {:error, _} if f.(v) results in an.Examplesiex&gt; map_all([&quot;23:50:07.0123456&quot;, &quot;23:50:07.123Z&quot;], &amp;Time.from_iso8601/1) {:ok, [~T[23:50:07.012345], ~T[23:50:07.123]]} iex&gt; map_all([&quot;23:50:61&quot;, &quot;23:50:07.123Z&quot;], &amp;Time.from_iso8601/1) {:error, :invalid_time}","ref":"Noether.Either.html#map_all/2","title":"Noether.Either.map_all/2","type":"function"},{"doc":"Given an Either and one function, it applies the function to the {:error, _} tuple.Examplesiex&gt; map_error({:ok, 1}, &amp;(&amp;1 + 1)) {:ok, 1} iex&gt; map_error({:error, 1}, &amp;(&amp;1 + 1)) {:error, 2}","ref":"Noether.Either.html#map_error/2","title":"Noether.Either.map_error/2","type":"function"},{"doc":"It returns true only if the value given matches a {:ok, value} type.Examplesiex&gt; ok?({:ok, 1}) true iex&gt; ok?({:error, 2}) false iex&gt; ok?(3) false","ref":"Noether.Either.html#ok?/1","title":"Noether.Either.ok?/1","type":"function"},{"doc":"Given a value and a function, it applies the function on the value returning {:ok, f.(value)}. If the function throws an exception e then it is wrapped into an {:error, e}.Examplesiex&gt; try(&quot;42&quot;, &amp;String.to_integer/1) {:ok, 42} iex&gt; try(&quot;nan&quot;, &amp;String.to_integer/1) {:error, %ArgumentError{message: &quot;argument error&quot;}}","ref":"Noether.Either.html#try/2","title":"Noether.Either.try/2","type":"function"},{"doc":"It returns the value of an {:ok, value} only if such a tuple is given. If not, the default value (nil if not provided) is returned.Examplesiex&gt; unwrap({:ok, 1}) 1 iex&gt; unwrap(2) nil iex&gt; unwrap({:ok, 1}, :default_value) 1 iex&gt; unwrap(2, :default_value) :default_value","ref":"Noether.Either.html#unwrap/2","title":"Noether.Either.unwrap/2","type":"function"},{"doc":"Given any value, it makes sure the result is an Either type.Examplesiex&gt; wrap({:ok, 1}) {:ok, 1} iex&gt; wrap({:error, 2}) {:error, 2} iex&gt; wrap(3) {:ok, 3}","ref":"Noether.Either.html#wrap/1","title":"Noether.Either.wrap/1","type":"function"},{"doc":"Given any value, it makes sure the result is an Either type.Examplesiex&gt; wrap_err({:ok, 1}) {:ok, 1} iex&gt; wrap_err({:error, 2}) {:error, 2} iex&gt; wrap_err(3) {:error, 3}","ref":"Noether.Either.html#wrap_err/1","title":"Noether.Either.wrap_err/1","type":"function"},{"doc":"","ref":"Noether.Either.html#t:either/0","title":"Noether.Either.either/0","type":"type"},{"doc":"","ref":"Noether.Either.html#t:fun1/0","title":"Noether.Either.fun1/0","type":"type"},{"doc":"","ref":"Noether.List.html","title":"Noether.List","type":"module"},{"doc":"Given a list, it returns {:ok, list} if every element of the list is different from nil or {:error, _}. Otherwise {:error, :nil_found} is returned.Examplesiex&gt; sequence([1, 2]) {:ok, [1, 2]} iex&gt; sequence([1, nil, 3]) {:error, :nil_found} iex&gt; sequence([{:ok, 1}, {:ok, 2}]) {:ok, [1, 2]} iex&gt; sequence([{:ok, 1}, {:error, 2}, {:ok, 3}]) {:error, 2} iex&gt; sequence([{:error, 1}, {:error, 2}]) {:error, 1}","ref":"Noether.List.html#sequence/1","title":"Noether.List.sequence/1","type":"function"},{"doc":"Given a predicate, a function of arity 1, and a value, the function is applied repeatedly until the predicate applied to the value returns either nil, false, or {:error, _}. The list of results is returned.Examplesiex&gt; until(fn a -&gt; a &lt; 10 end, &amp;(&amp;1 + 1), 0) [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]","ref":"Noether.List.html#until/3","title":"Noether.List.until/3","type":"function"},{"doc":"Given two lists and a function of arity 2, the lists are first zipped and then each tuple is applied (curried) to the function.Examplesiex&gt; zip_with([1, 2, 3], [4, 5, 6], &amp;Kernel.+/2) [5, 7, 9]","ref":"Noether.List.html#zip_with/3","title":"Noether.List.zip_with/3","type":"function"},{"doc":"","ref":"Noether.List.html#t:fun1/0","title":"Noether.List.fun1/0","type":"type"},{"doc":"","ref":"Noether.List.html#t:fun2/0","title":"Noether.List.fun2/0","type":"type"},{"doc":"","ref":"Noether.Maybe.html","title":"Noether.Maybe","type":"module"},{"doc":"Given a list of values, the function is mapped only on the elements different from nil. nil values will be discarded. A list of the results is returned.Examplesiex&gt; cat_maybe([1], &amp;(&amp;1 + 1)) [2] iex&gt; cat_maybe([1, nil, 3], &amp;(&amp;1 + 1)) [2, 4]","ref":"Noether.Maybe.html#cat_maybe/2","title":"Noether.Maybe.cat_maybe/2","type":"function"},{"doc":"Given a value and two functions, it applies the first one and returns the result if it's different from nil. Otherwise the second function is applied.Examplesiex&gt; choose(0, fn a -&gt; a + 1 end, fn b -&gt; b + 2 end) 1 iex&gt; choose(0, fn _ -&gt; nil end, fn b -&gt; b + 2 end) 2 iex&gt; choose(0, fn _ -&gt; nil end, fn _ -&gt; nil end) nil","ref":"Noether.Maybe.html#choose/3","title":"Noether.Maybe.choose/3","type":"function"},{"doc":"Given a value and a function, the function is applied only if the value is different from nil. nil is returned otherwise.Examplesiex&gt; map(nil, &amp;Kernel.abs/1) nil iex&gt; map(-1, &amp;Kernel.abs/1) 1","ref":"Noether.Maybe.html#map/2","title":"Noether.Maybe.map/2","type":"function"},{"doc":"Given a value, a function, and a default, it applies the function on the value if the latter is different from nil. It returns the default otherwise.Examplesiex&gt; maybe(-1, &amp;Kernel.abs/1, :hello) 1 iex&gt; maybe(nil, &amp;Kernel.abs/1, :hello) :hello","ref":"Noether.Maybe.html#maybe/3","title":"Noether.Maybe.maybe/3","type":"function"},{"doc":"Given a value and a default, {:ok, value} is returned only if the value is different from nil. {:error, default} is returned otherwise.Examplesiex&gt; required(nil, :hello) {:error, :hello} iex&gt; required(1, :hello) {:ok, 1}","ref":"Noether.Maybe.html#required/2","title":"Noether.Maybe.required/2","type":"function"},{"doc":"","ref":"Noether.Maybe.html#t:fun1/0","title":"Noether.Maybe.fun1/0","type":"type"}]